from typing import ClassVar, Optional

from google.protobuf.json_format import MessageToDict
from grpc.aio import insecure_channel
from pydantic import BaseModel, validate_call

from .api_constants import (
    SUCCESS_CODE,
    ControllerStubs,
    Entities,
    RequestMessages,
    RPCMethodNames,
)


class ValidationError(Exception):
    pass


class ResponseCodeError(Exception):
    pass


class Base(BaseModel):
    """
    Configure the host and port of the search server.

    Parameters:
        host (str): The host address of the search server.
        port (int | str): The port number of the search server.
        api_key (str): The api_key of the search server.
        place (str): The place you want to makes operations about.
        tag (str): The tag of the place you want to work with.
    """

    host: str = "127.0.0.1"
    port: str | int = "50051"
    api_key: str
    place: Optional[str] = None
    tag: Optional[str] = None

    __instance: ClassVar = None

    def __new__(cls, *args, **kwargs):
        if cls.__instance is None:
            cls.__instance = super().__new__(cls)
        return cls.__instance

    async def perform_operation(self, rpc_method_name, request_message):
        """Perform the operation.

        Parameters:
            rpc_method_name (str): The name of the RPC method (e.g., `Create`).
            request_message (gRPC message): The request message instance.
        """
        async with insecure_channel(f"{self.host}:{self.port}") as channel:
            self.controller_stub.channel = channel
            self.controller_stub.ready()
            RPCMethod = getattr(self.controller_stub, rpc_method_name, None)

            if RPCMethod:
                metadata = [("api-key", self.api_key)]
                response = await RPCMethod(request_message, metadata=metadata)

                # Clear the request message for future use
                request_message.Clear()

                # Raise error if response is not successful
                if response.code != SUCCESS_CODE:
                    raise ResponseCodeError(response.code)

                return MessageToDict(response, preserving_proto_field_name=True)
            else:
                raise ValidationError("Invalid RPC method name.")

    @validate_call
    async def search(
        self,
        place: Optional[str] = None,
        tag: Optional[str] = None,
    ):
        """Search for records with provided `place` and `tag` from a database model."""
        # Get place, tag from self if aren't provided here
        place, tag = place or self.place, tag or self.tag

        # Assign attributes
        request = RequestMessages.SEARCH(place=place, tag=tag)

        return await self.perform_operation(RPCMethodNames.SEARCH, request)


class Media(Base):
    """`search()`, `create()`."""

    controller_stub: ClassVar = ControllerStubs.MEDIA

    @validate_call
    async def create(
        self,
        media_type: str,
        media_identifier: str,
        place: Optional[str] = None,
        tag: Optional[str] = None,
    ):
        # Get place, tag from self if aren't provided here
        place, tag = place or self.place, tag or self.tag

        # Assign attributes
        RequestMessages.ADD_MEDIA.media_type = media_type
        RequestMessages.ADD_MEDIA.media_type = media_identifier
        RequestMessages.ADD_MEDIA.media_type = place
        RequestMessages.ADD_MEDIA.media_type = tag

        return await self.perform_operation(
            RPCMethodNames.CREATE, RequestMessages.ADD_MEDIA
        )


class Content(Base):
    """`search()`, `create()`."""

    controller_stub: ClassVar = ControllerStubs.CONTENT

    @validate_call
    async def create(
        self,
        header: str,
        content: str,
        order: Optional[int] = None,
        place: Optional[str] = None,
        tag: Optional[str] = None,
    ):
        # Get place, tag from self if aren't provided here
        place, tag = place or self.place, tag or self.tag

        # Assign attributes
        if order:
            RequestMessages.ADD_MEDIA.order = order
        RequestMessages.ADD_MEDIA.header = header
        RequestMessages.ADD_MEDIA.content = content
        RequestMessages.ADD_MEDIA.place = place
        RequestMessages.ADD_MEDIA.media_type = tag

        return await self.perform_operation(
            RPCMethodNames.CREATE, RequestMessages.ADD_CONTENT
        )


class Seo(Base):
    """`search()`, `create()`."""

    controller_stub: ClassVar = ControllerStubs.SEO

    @validate_call
    async def create(
        self,
        seo_title: str,
        url: str,
        canonical_tag: str,
        meta_description: str,
        no_index: bool,
        no_follow: bool,
        og_image_media_identifier: str,
        og_image_width: int,
        og_image_height: int,
        place: Optional[str] = None,
        tag: Optional[str] = None,
    ):
        # Get place, tag from self if aren't provided here
        place, tag = place or self.place, tag or self.tag

        # Assign attributes
        RequestMessages.ADD_SEO.seo_title = seo_title
        RequestMessages.ADD_SEO.url = url
        RequestMessages.ADD_SEO.canonical_tag = canonical_tag
        RequestMessages.ADD_SEO.meta_description = meta_description
        RequestMessages.ADD_SEO.visit_page = og_image_media_identifier
        RequestMessages.ADD_SEO.register_user_identifier = og_image_width
        RequestMessages.ADD_SEO.seo_page_type = og_image_height
        RequestMessages.ADD_SEO.no_follow = no_follow
        RequestMessages.ADD_SEO.no_index = no_index
        RequestMessages.ADD_SEO.place = place
        RequestMessages.ADD_SEO.tag = tag

        return await self.perform_operation(
            RPCMethodNames.CREATE, RequestMessages.ADD_SEO
        )

    @validate_call
    async def get_blogs_sitemap(self):
        request = RequestMessages.SITEMAP(entity=Entities.BLOG)
        return await self.perform_operation(RPCMethodNames.GET_SITEMAP, request)

    @validate_call
    async def get_courses_sitemap(self):
        request = RequestMessages.SITEMAP(entity=Entities.COURSE)
        return await self.perform_operation(RPCMethodNames.GET_SITEMAP, request)

    @validate_call
    async def get_lessons_sitemap(self):
        request = RequestMessages.SITEMAP(entity=Entities.LESSON)
        return await self.perform_operation(RPCMethodNames.GET_SITEMAP, request)

    @validate_call
    async def get_master_courses_sitemap(self):
        request = RequestMessages.SITEMAP(entity=Entities.MASTER_COURSE)
        return await self.perform_operation(RPCMethodNames.GET_SITEMAP, request)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: place_tag/grpc/place_tag.proto
# Protobuf Python Version: 4.25.1
# pylint: skip-file
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x1eplace_tag/grpc/place_tag.proto\x12\x10\x63onfig.place_tag\x1a\x1cgoogle/protobuf/struct.proto"k\n\x0e\x43ontentRequest\x12\r\n\x05place\x18\x01 \x01(\t\x12\x0b\n\x03tag\x18\x02 \x01(\t\x12\x12\n\x05order\x18\x03 \x01(\x05H\x00\x88\x01\x01\x12\x0e\n\x06header\x18\x04 \x01(\t\x12\x0f\n\x07\x63ontent\x18\x05 \x01(\tB\x08\n\x06_order"W\n\x0f\x43ontentResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.Struct"2\n\x14\x43ontentSearchRequest\x12\r\n\x05place\x18\x01 \x01(\t\x12\x0b\n\x03tag\x18\x02 \x01(\t"X\n\x0cMediaRequest\x12\r\n\x05place\x18\x01 \x01(\t\x12\x0b\n\x03tag\x18\x02 \x01(\t\x12\x12\n\nmedia_type\x18\x03 \x01(\t\x12\x18\n\x10media_identifier\x18\x04 \x01(\t"U\n\rMediaResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.Struct"0\n\x12MediaSearchRequest\x12\r\n\x05place\x18\x01 \x01(\t\x12\x0b\n\x03tag\x18\x02 \x01(\t"\xbb\x03\n\nSeoRequest\x12\r\n\x05place\x18\x01 \x01(\t\x12\x0b\n\x03tag\x18\x02 \x01(\t\x12\x12\n\x05title\x18\x03 \x01(\tH\x00\x88\x01\x01\x12\x15\n\rcanonical_tag\x18\x07 \x01(\t\x12\x18\n\x10meta_description\x18\x08 \x01(\t\x12\x33\n\rmeta_keywords\x18\t \x01(\x0b\x32\x17.google.protobuf.StructH\x01\x88\x01\x01\x12\x10\n\x08no_index\x18\x0f \x01(\x08\x12\x12\n\npage_title\x18\x10 \x01(\t\x12\'\n\x1apage_icon_media_identifier\x18\x11 \x01(\tH\x02\x88\x01\x01\x12\x11\n\tno_follow\x18\x13 \x01(\x08\x12\x16\n\x0eog_image_width\x18\x15 \x01(\x05\x12\x17\n\x0fog_image_height\x18\x16 \x01(\x05\x12!\n\x19og_image_media_identifier\x18\x17 \x01(\t\x12\x17\n\nfront_path\x18\x18 \x01(\tH\x03\x88\x01\x01\x42\x08\n\x06_titleB\x10\n\x0e_meta_keywordsB\x1d\n\x1b_page_icon_media_identifierB\r\n\x0b_front_path"S\n\x0bSeoResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.Struct".\n\x10SeoSearchRequest\x12\r\n\x05place\x18\x01 \x01(\t\x12\x0b\n\x03tag\x18\x02 \x01(\t" \n\x0eSitemapRequest\x12\x0e\n\x06\x65ntity\x18\x01 \x01(\t"W\n\x0fSitemapResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.Struct2\xbb\x01\n\x11\x43ontentController\x12O\n\x06\x43reate\x12 .config.place_tag.ContentRequest\x1a!.config.place_tag.ContentResponse"\x00\x12U\n\x06Search\x12&.config.place_tag.ContentSearchRequest\x1a!.config.place_tag.ContentResponse"\x00\x32\xb1\x01\n\x0fMediaController\x12K\n\x06\x43reate\x12\x1e.config.place_tag.MediaRequest\x1a\x1f.config.place_tag.MediaResponse"\x00\x12Q\n\x06Search\x12$.config.place_tag.MediaSearchRequest\x1a\x1f.config.place_tag.MediaResponse"\x00\x32\xfc\x01\n\rSeoController\x12G\n\x06\x43reate\x12\x1c.config.place_tag.SeoRequest\x1a\x1d.config.place_tag.SeoResponse"\x00\x12S\n\nGetSitemap\x12 .config.place_tag.SitemapRequest\x1a!.config.place_tag.SitemapResponse"\x00\x12M\n\x06Search\x12".config.place_tag.SeoSearchRequest\x1a\x1d.config.place_tag.SeoResponse"\x00\x62\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(
    DESCRIPTOR, "place_tag.grpc.place_tag_pb2", _globals
)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _globals["_CONTENTREQUEST"]._serialized_start = 82
    _globals["_CONTENTREQUEST"]._serialized_end = 189
    _globals["_CONTENTRESPONSE"]._serialized_start = 191
    _globals["_CONTENTRESPONSE"]._serialized_end = 278
    _globals["_CONTENTSEARCHREQUEST"]._serialized_start = 280
    _globals["_CONTENTSEARCHREQUEST"]._serialized_end = 330
    _globals["_MEDIAREQUEST"]._serialized_start = 332
    _globals["_MEDIAREQUEST"]._serialized_end = 420
    _globals["_MEDIARESPONSE"]._serialized_start = 422
    _globals["_MEDIARESPONSE"]._serialized_end = 507
    _globals["_MEDIASEARCHREQUEST"]._serialized_start = 509
    _globals["_MEDIASEARCHREQUEST"]._serialized_end = 557
    _globals["_SEOREQUEST"]._serialized_start = 560
    _globals["_SEOREQUEST"]._serialized_end = 1003
    _globals["_SEORESPONSE"]._serialized_start = 1005
    _globals["_SEORESPONSE"]._serialized_end = 1088
    _globals["_SEOSEARCHREQUEST"]._serialized_start = 1090
    _globals["_SEOSEARCHREQUEST"]._serialized_end = 1136
    _globals["_SITEMAPREQUEST"]._serialized_start = 1138
    _globals["_SITEMAPREQUEST"]._serialized_end = 1170
    _globals["_SITEMAPRESPONSE"]._serialized_start = 1172
    _globals["_SITEMAPRESPONSE"]._serialized_end = 1259
    _globals["_CONTENTCONTROLLER"]._serialized_start = 1262
    _globals["_CONTENTCONTROLLER"]._serialized_end = 1449
    _globals["_MEDIACONTROLLER"]._serialized_start = 1452
    _globals["_MEDIACONTROLLER"]._serialized_end = 1629
    _globals["_SEOCONTROLLER"]._serialized_start = 1632
    _globals["_SEOCONTROLLER"]._serialized_end = 1884
# @@protoc_insertion_point(module_scope)



from .grpc_lib.place_tag_pb2 import (
    ContentRequest,
    MediaRequest,
    SeoRequest,
    SeoSearchRequest,
    SitemapRequest,
)
from .grpc_lib.place_tag_pb2_grpc import (
    ContentControllerStub,
    MediaControllerStub,
    SeoControllerStub,
)

SUCCESS_CODE = "0"


class RequestMessages:
    ADD_MEDIA = MediaRequest
    ADD_CONTENT = ContentRequest
    ADD_SEO = SeoRequest
    SITEMAP = SitemapRequest
    # search messages are all the same so we use one
    SEARCH = SeoSearchRequest


class RPCMethodNames:
    CREATE = "Create"
    SEARCH = "Search"
    GET_SITEMAP = "GetSitemap"


class ControllerStubs:
    MEDIA = MediaControllerStub()
    CONTENT = ContentControllerStub()
    SEO = SeoControllerStub()


class Entities:
    COURSE = "course"
    LESSON = "lesson"
    MASTER_COURSE = "master_course"
    BLOG = "blog"



# pylint: skip-file
# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""

import grpc
from . import place_tag_pb2 as place__tag_dot_grpc_dot_place__tag__pb2


class ContentControllerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel=None):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.channel = channel
        self.ready()

    def ready(self):
        if self.channel is not None:
            self.Create = self.channel.unary_unary(
                "/config.place_tag.ContentController/Create",
                request_serializer=place__tag_dot_grpc_dot_place__tag__pb2.ContentRequest.SerializeToString,
                response_deserializer=place__tag_dot_grpc_dot_place__tag__pb2.ContentResponse.FromString,
            )
            self.Search = self.channel.unary_unary(
                "/config.place_tag.ContentController/Search",
                request_serializer=place__tag_dot_grpc_dot_place__tag__pb2.ContentSearchRequest.SerializeToString,
                response_deserializer=place__tag_dot_grpc_dot_place__tag__pb2.ContentResponse.FromString,
            )


class ContentControllerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Create(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def Search(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_ContentControllerServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "Create": grpc.unary_unary_rpc_method_handler(
            servicer.Create,
            request_deserializer=place__tag_dot_grpc_dot_place__tag__pb2.ContentRequest.FromString,
            response_serializer=place__tag_dot_grpc_dot_place__tag__pb2.ContentResponse.SerializeToString,
        ),
        "Search": grpc.unary_unary_rpc_method_handler(
            servicer.Search,
            request_deserializer=place__tag_dot_grpc_dot_place__tag__pb2.ContentSearchRequest.FromString,
            response_serializer=place__tag_dot_grpc_dot_place__tag__pb2.ContentResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "config.place_tag.ContentController", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class ContentController(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Create(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.place_tag.ContentController/Create",
            place__tag_dot_grpc_dot_place__tag__pb2.ContentRequest.SerializeToString,
            place__tag_dot_grpc_dot_place__tag__pb2.ContentResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def Search(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.place_tag.ContentController/Search",
            place__tag_dot_grpc_dot_place__tag__pb2.ContentSearchRequest.SerializeToString,
            place__tag_dot_grpc_dot_place__tag__pb2.ContentResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )


class MediaControllerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel=None):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.channel = channel
        self.ready()

    def ready(self):
        if self.channel is not None:
            self.Create = self.channel.unary_unary(
                "/config.place_tag.MediaController/Create",
                request_serializer=place__tag_dot_grpc_dot_place__tag__pb2.MediaRequest.SerializeToString,
                response_deserializer=place__tag_dot_grpc_dot_place__tag__pb2.MediaResponse.FromString,
            )
            self.Search = self.channel.unary_unary(
                "/config.place_tag.MediaController/Search",
                request_serializer=place__tag_dot_grpc_dot_place__tag__pb2.MediaSearchRequest.SerializeToString,
                response_deserializer=place__tag_dot_grpc_dot_place__tag__pb2.MediaResponse.FromString,
            )


class MediaControllerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Create(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def Search(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_MediaControllerServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "Create": grpc.unary_unary_rpc_method_handler(
            servicer.Create,
            request_deserializer=place__tag_dot_grpc_dot_place__tag__pb2.MediaRequest.FromString,
            response_serializer=place__tag_dot_grpc_dot_place__tag__pb2.MediaResponse.SerializeToString,
        ),
        "Search": grpc.unary_unary_rpc_method_handler(
            servicer.Search,
            request_deserializer=place__tag_dot_grpc_dot_place__tag__pb2.MediaSearchRequest.FromString,
            response_serializer=place__tag_dot_grpc_dot_place__tag__pb2.MediaResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "config.place_tag.MediaController", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class MediaController(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Create(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.place_tag.MediaController/Create",
            place__tag_dot_grpc_dot_place__tag__pb2.MediaRequest.SerializeToString,
            place__tag_dot_grpc_dot_place__tag__pb2.MediaResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def Search(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.place_tag.MediaController/Search",
            place__tag_dot_grpc_dot_place__tag__pb2.MediaSearchRequest.SerializeToString,
            place__tag_dot_grpc_dot_place__tag__pb2.MediaResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )


class SeoControllerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel=None):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.channel = channel
        self.ready()

    def ready(self):
        if self.channel is not None:
            self.Create = self.channel.unary_unary(
                "/config.place_tag.SeoController/Create",
                request_serializer=place__tag_dot_grpc_dot_place__tag__pb2.SeoRequest.SerializeToString,
                response_deserializer=place__tag_dot_grpc_dot_place__tag__pb2.SeoResponse.FromString,
            )
            self.GetSitemap = self.channel.unary_unary(
                "/config.place_tag.SeoController/GetSitemap",
                request_serializer=place__tag_dot_grpc_dot_place__tag__pb2.SitemapRequest.SerializeToString,
                response_deserializer=place__tag_dot_grpc_dot_place__tag__pb2.SitemapResponse.FromString,
            )
            self.Search = self.channel.unary_unary(
                "/config.place_tag.SeoController/Search",
                request_serializer=place__tag_dot_grpc_dot_place__tag__pb2.SeoSearchRequest.SerializeToString,
                response_deserializer=place__tag_dot_grpc_dot_place__tag__pb2.SeoResponse.FromString,
            )


class SeoControllerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Create(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def GetSitemap(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def Search(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_SeoControllerServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "Create": grpc.unary_unary_rpc_method_handler(
            servicer.Create,
            request_deserializer=place__tag_dot_grpc_dot_place__tag__pb2.SeoRequest.FromString,
            response_serializer=place__tag_dot_grpc_dot_place__tag__pb2.SeoResponse.SerializeToString,
        ),
        "GetSitemap": grpc.unary_unary_rpc_method_handler(
            servicer.GetSitemap,
            request_deserializer=place__tag_dot_grpc_dot_place__tag__pb2.SitemapRequest.FromString,
            response_serializer=place__tag_dot_grpc_dot_place__tag__pb2.SitemapResponse.SerializeToString,
        ),
        "Search": grpc.unary_unary_rpc_method_handler(
            servicer.Search,
            request_deserializer=place__tag_dot_grpc_dot_place__tag__pb2.SeoSearchRequest.FromString,
            response_serializer=place__tag_dot_grpc_dot_place__tag__pb2.SeoResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "config.place_tag.SeoController", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class SeoController(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Create(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.place_tag.SeoController/Create",
            place__tag_dot_grpc_dot_place__tag__pb2.SeoRequest.SerializeToString,
            place__tag_dot_grpc_dot_place__tag__pb2.SeoResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def GetSitemap(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.place_tag.SeoController/GetSitemap",
            place__tag_dot_grpc_dot_place__tag__pb2.SitemapRequest.SerializeToString,
            place__tag_dot_grpc_dot_place__tag__pb2.SitemapResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def Search(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.place_tag.SeoController/Search",
            place__tag_dot_grpc_dot_place__tag__pb2.SeoSearchRequest.SerializeToString,
            place__tag_dot_grpc_dot_place__tag__pb2.SeoResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )
